/*
Copyright (C) 2004 Observatoire de Paris-Meudon

Ce programme est un logiciel libre ; vous pouvez le redistribuer et/ou le modifier conformment aux dispositions de la Licence Publique Gnrale GNU, telle que publie par la Free Software Foundation ; version 2 de la licence, ou encore ( votre choix) toute version ultrieure.

Ce programme est distribu dans l'espoir qu'il sera utile, mais SANS AUCUNE GARANTIE ; sans mme la garantie implicite de COMMERCIALISATION ou D'ADAPTATION A UN OBJET PARTICULIER. Pour plus de dtail, voir la Licence Publique Gnrale GNU .

Vous devez avoir reu un exemplaire de la Licence Publique Gnrale GNU en mme temps que ce programme ; si ce n'est pas le cas, crivez  la Free Software Foundation Inc., 675 Mass Ave, Cambridge, MA 02139, Etats-Unis.
*/

package corpsnoir;

import java.applet.Applet;
import java.awt.*;
import java.awt.event.*;
import java.util.ResourceBundle;

public class CorpsNoir extends Applet implements AdjustmentListener, ItemListener {
    
    static final ResourceBundle rb  = ResourceBundle.getBundle("corpsnoir.LocalStrings");
    static final int imgdimx = 300;
    static final int imgdimy = 300;
    static final double minTemperature = 300;
    static final double maxTemperature = 30000;
    static final int nbptstemp = 50;
    static final double minLambda = 0.0;//0.1E-6;
    protected double maxLambda = 1E-6;
    static final double c = 2.99792958E8;
    static final double h = 6.6260755E-34;
    static final double k = 1.380622E-23;
    //static final double wien = 4.79E-3;
    static final double wien = 2.89E-3;
    static final int espx = 15, espy = 17;
    protected ImageCanvas ic;
    protected Image croquis = null;
    protected Graphics g = null;
    protected Scrollbar sliderTemperature;
    protected double temperature;
    protected Label labelTemperature;
    protected Label labelLambdaMax;
    protected Checkbox checkNormaliser;
    protected Checkbox checkLog;
    protected Choice choixMax;
    protected boolean normaliser = true;
    protected boolean yLog = false;
    protected Color[] couleurs;
    
    public void init() {
        setLayout(new BorderLayout());
        
        creerCroquis();
        ic = new ImageCanvas(imgdimx, imgdimy);
        add(ic, BorderLayout.CENTER);
        
        Panel hautPane = new Panel();
        checkNormaliser = new Checkbox(rb.getString("normaliser"), normaliser);
        checkNormaliser.addItemListener(this);
        hautPane.add(checkNormaliser);
        checkLog = new Checkbox(rb.getString("chelle_log"), yLog);
        checkLog.addItemListener(this);
        hautPane.add(checkLog);
        hautPane.add(new Label(rb.getString("max")));
        choixMax = new Choice();
        choixMax.add("1 m");
        choixMax.add("10 m");
        choixMax.addItemListener(this);
        hautPane.add(choixMax);
        add(hautPane, BorderLayout.NORTH);
        
        Panel temperaturePane = new Panel(new GridLayout(7,1)); // le dernier sert de marge en bas
        add(temperaturePane, BorderLayout.SOUTH);
        
        temperaturePane.add(new Label(" "));
        labelTemperature = new Label();
        temperaturePane.add(labelTemperature);
        int itemp = 2*nbptstemp/3;
        sliderTemperature = new Scrollbar(Scrollbar.HORIZONTAL, itemp, 3, 1, 3+nbptstemp);
        temperature = Math.pow(10, Math.log(minTemperature)/Math.log(10) +
            Math.log(maxTemperature/minTemperature)/Math.log(10) * (itemp - 1)/(nbptstemp - 1.0));
        labelTemperature.setText(rb.getString("Temprature") + " " + (int)Math.round(temperature) + " K");
        sliderTemperature.setBlockIncrement(4);
        sliderTemperature.addAdjustmentListener(this);
        temperaturePane.add(sliderTemperature);
        Panel valeursTemperaturePane = new Panel(new GridLayout(1,5));
        valeursTemperaturePane.add(new Label("300K", Label.CENTER));
        valeursTemperaturePane.add(new Label("1000K", Label.CENTER));
        valeursTemperaturePane.add(new Label("3000K", Label.CENTER));
        valeursTemperaturePane.add(new Label("10000K", Label.CENTER));
        valeursTemperaturePane.add(new Label("30000K", Label.CENTER));
        temperaturePane.add(valeursTemperaturePane);
        temperaturePane.add(new Label(" "));
        labelLambdaMax = new Label(" ");
        temperaturePane.add(labelLambdaMax);
        
        initTableCouleurs();
        
        majCroquis();
        
        validate();
        if (ic != null)
            ic.requestFocus();
    }
    
    protected void initTableCouleurs() {
        int npix = imgdimx - 2 * espx;
        couleurs = new Color[npix];
        for (int i=0; i<npix; i++) {
            double lambda = minLambda + ((maxLambda - minLambda)*i)/npix;
            // couleurs issues de
            // http://www.efg2.com/Lab/ScienceAndEngineering/Spectra.htm
            // et http://www.physics.sfasu.edu/astro/color.html
            double w2 = lambda * 1E9;
            double r1 , v1, b1;
            if (w2 >= 380 && w2 < 440) {
                r1 = - (w2 - 440) / (440 - 380);
                v1 = 0;
                b1 = 1;
            } else if (w2 >= 440 && w2 < 490) {
                r1 = 0;
                v1 = (w2 - 440) / (490 - 440);
                b1 = 1;
            } else if (w2 >= 490 && w2 < 510) {
                r1 = 0;
                v1 = 1;
                b1 = - (w2 - 510) / (510 - 490);
            } else if (w2 >= 510 && w2 < 580) {
                r1 = (w2 - 510) / (580 - 510);
                v1 = 1;
                b1 = 0;
            } else if (w2 >= 580 && w2 < 645) {
                r1 = 1;
                v1 = -(w2 - 645) / (645 - 580);
                b1 = 0;
            } else if (w2 >= 645 && w2 < 780) {
                r1 = 1;
                v1 = 0;
                b1 = 0;
            } else {
                r1 = 0;
                v1 = 0;
                b1 = 0;
            }
            double intens;
            if (w2 >= 380 && w2 < 420)
                intens = 0.3 + 0.7*(w2 - 380) / (420 - 380);
            else if (w2 >= 420 && w2 < 700)
                intens = 1;
            else if (w2 >= 700 && w2 < 780)
                intens = 0.3 + 0.7*(780 - w2) / (780 - 700);
            else
                intens = 0;
            int rc = (int)Math.round(255*r1*intens);
            int vc = (int)Math.round(255*v1*intens);
            int bc = (int)Math.round(255*b1*intens);
            couleurs[i] = new Color(rc, vc, bc);
        }
    }
    
    public void adjustmentValueChanged(AdjustmentEvent e) {
        Adjustable adj = e.getAdjustable();
        if (adj == sliderTemperature) {
            double ntemperature = Math.pow(10, Math.log(minTemperature)/Math.log(10) +
                Math.log(maxTemperature/minTemperature)/Math.log(10) * (e.getValue()-1)/(nbptstemp - 1.0));
            if (ntemperature != temperature) {
                temperature = ntemperature;
                labelTemperature.setText(rb.getString("Temprature") + " " + (int)Math.round(temperature) + " K");
                majCroquis();
            }
        }
    }
    
    public void itemStateChanged(ItemEvent e) {
        if (e.getSource() == checkNormaliser) {
            normaliser = checkNormaliser.getState();
            majCroquis();
        } else if (e.getSource() == checkLog) {
            yLog = checkLog.getState();
            majCroquis();
        } else if (e.getSource() == choixMax) {
            String svmax = choixMax.getSelectedItem();
            if ("1 m".equals(svmax))
                maxLambda = 1E-6;
            else if ("10 m".equals(svmax))
                maxLambda = 1E-5;
            initTableCouleurs();
            majCroquis();
        }
    }
    
    protected void creerCroquis() {
        croquis = createImage(imgdimx, imgdimy);
        g = croquis.getGraphics();
        g.setFont(new Font(getFont().getName(), Font.BOLD, 12));
    }
    
    protected void majCroquis() {
        g.setColor(Color.white);
        g.fillRect(0, 0, imgdimx, imgdimy);
        g.setColor(Color.black);
        g.drawRect(0, 0, imgdimx-1, imgdimy-1);
        
        g.drawLine(espx, 5, espx, imgdimy - espy);
        g.drawLine(espx, 5, espx - 2, 8);
        g.drawLine(espx, 5, espx + 2, 8);
        g.drawLine(espx, imgdimy - espy, imgdimx - espx, imgdimy - espy);
        g.drawLine(imgdimx - espx, imgdimy - espy, imgdimx - espx - 3, imgdimy - espy - 2);
        g.drawLine(imgdimx - espx, imgdimy - espy, imgdimx - espx - 3, imgdimy - espy + 2);
        g.drawString("1", 2, 15);
        g.drawString("0", 2, imgdimy - espy);
        g.drawString(joliFormattage(minLambda*1E6)+"\u03BCm", espx, imgdimy - 4);
        g.drawString(joliFormattage(maxLambda*1E6)+"\u03BCm", imgdimx - 30, imgdimy - 4);
        g.drawString("\u03BB", imgdimx - espx + 2, imgdimy - espy);
        
        int npix = imgdimx - 2 * espx;
        double huitpihc = 8*Math.PI*h*c;
        double hsurk = h/k;
        double umax;
        double lambdamax = wien/temperature;
        labelLambdaMax.setText("\u03BB max = "+ joliFormattage(1E6*lambdamax) + " \u03BCm");
        if (normaliser) {
            //double numax = temperature*c/wien;
            //umax = huitpihc*Math.pow(numax, 3)/(Math.exp(hsurk*numax/temperature) - 1);
            umax = Math.pow(lambdamax, -5)/(Math.exp(hsurk*c/(lambdamax*temperature)) - 1);
        } else {
            //double numax = maxTemperature*c/wien;
            //umax = huitpihc*Math.pow(numax, 3)/(Math.exp(hsurk*numax/maxTemperature) - 1);
            lambdamax = wien/maxTemperature;
            umax = Math.pow(lambdamax, -5)/(Math.exp(hsurk*c/(lambdamax*maxTemperature)) - 1);
        }
        if (yLog)
            umax = Math.log(umax);
        for (int i=0; i<npix; i++) {
            double lambda = minLambda + ((maxLambda - minLambda)*i)/npix;
            double nu = c / lambda;
            //double u = huitpihc*Math.pow(nu, 3)/(Math.exp(hsurk*nu/temperature) - 1);
            double u = Math.pow(lambda, -5)/(Math.exp(hsurk*c/(lambda*temperature)) - 1);
            if (yLog)
                u = Math.log(u);
            if (u/umax > 1E-3) {
                int y1 = imgdimy - espy - (int)Math.round((imgdimy - espy - 5)*(u/umax));
                g.setColor(couleurs[i]);
                g.drawLine(espx + i, y1, espx + i, imgdimy - espy - 1);
            }
        }
        
        ic.updateImage(croquis);
    }
    
	// mthode d'affichage de double issue du Calcotron
    public static String joliFormattage(double v) {
        int precision = 3;
        int limitesci = 3; // dcimales avant de passer en mode sci
        
        if (v == 0)
            return("0");
        String sj = ""+v;
        if ("Infinity".equals(sj) || "-Infinity".equals(sj) || "NaN".equals(sj))
            return(""+v);
        String signe;
        if (v >= 0)
            signe = "";
        else {
            v = -v;
            signe = "-";
        }
        int exposant = (int)Math.floor(Math.log(v)/Math.log(10));
        double exparr = Math.pow(10, precision);
        double mantisse = v / Math.pow(10, exposant);
        mantisse = Math.rint(mantisse*exparr)/exparr;
        if (mantisse >= 10) {
            exposant += 1;
            mantisse = v / Math.pow(10, exposant);
            mantisse = Math.rint(mantisse*exparr)/exparr;
        }
        String sv = "";
        int chiffre = (int)Math.floor(mantisse);
        sv += chiffre + ".";
        for (int i=0; i<precision-1; i++) {
            mantisse = (mantisse - chiffre)*10;
            double exparr2 = Math.pow(10, precision-i);
            mantisse = Math.rint(mantisse*exparr)/exparr;
            chiffre = (int)Math.floor(mantisse);
            sv += chiffre;
        }
        mantisse = (mantisse - chiffre)*10;
        chiffre = (int)Math.round(mantisse);
        sv += chiffre;
        char dernier = sv.charAt(sv.length()-1);
        while (dernier == '0') {
            sv = sv.substring(0, sv.length()-1);
            dernier = sv.charAt(sv.length()-1);
        }
        if (exposant != 0) {
            if (exposant > 0 && exposant < limitesci) {
                String sd = sv.substring(0,1);
                if (sv.length()-2 < exposant) {
                    // transformation de 3.2E2 en 320
                    if (sv.length() > 2)
                        sd += sv.substring(2,sv.length());
                    for (int i=0; i<exposant-(sv.length()-2); i++)
                        sd += '0';
                } else {
                    // transformation de 3.245E2 en 324.5
                    if (sv.length() < exposant+2)
                        sd += sv.substring(2);
                    else
                        sd += sv.substring(2,exposant+2) + "." + sv.substring(exposant+2);
                }
                sv = sd;
            } else if (exposant < 0 && exposant > -limitesci) {
                // transformation de 3.2E-2 en 0.032
                String sd = "0.";
                for (int i=0; i<-exposant-1; i++)
                    sd += '0';
                sv = sd + sv.charAt(0) + sv.substring(2);
            } else {
                if (sv.charAt(sv.length()-1) == '.')
                    sv = sv.substring(0, sv.length()-1);
                sv += ".10^" + exposant;
            }
        }
        if (sv.charAt(sv.length()-1) == '.')
            sv = sv.substring(0, sv.length()-1);
        
        sv = signe + sv;
        
        return(sv);
    }
    
}
