/*

Copyright (C) 2012  Observatoire de Paris

The JavaScript code in this page is free software: you can
redistribute it and/or modify it under the terms of the GNU
General Public License (GNU GPL) as published by the Free Software
Foundation, either version 3 of the License, or (at your option)
any later version.  The code is distributed WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU GPL for more details.

As additional permission under GNU GPL version 3 section 7, you
may distribute non-source (e.g., minimized or compacted) forms of
that code without the copy of the GNU GPL normally required by
section 4, provided you include this license notice and a URL
through which recipients can access the Corresponding Source.

*/

var p_lambda_max;
var dessin;
var context;
var imgdimx;
var imgdimy;
var espx = 15;
var espy = 17;
var minLambda = 0.0;
var maxLambda = 1E-6;
var c = 2.99792958E8;
var h = 6.6260755E-34;
var k = 1.380622E-23;
var wien = 2.89E-3;
var minTemperature = 300;
var maxTemperature = 30000;
var nbptstemp = 100;
var temperature = 3000;
var normaliser = true;
var yLog = false;
var degrade;
var curseur_temperature;
var idTimeout = 0;

function init() {
    dessin = document.getElementById('dessin');
    
    // Vérif que l'élément est dans le DOM et que le navigateur gère canvas
    if (dessin && dessin.getContext) {
        imgdimx = dessin.width;
        if (typeof imgdimx == 'string')
            imgdimx = parseInt(imgdimx);
        imgdimy = dessin.height;
        if (typeof imgdimy == 'string')
            imgdimy = parseInt(imgdimy);
        p_lambda_max = document.getElementById('lambda_max');
        curseur_temperature = new Curseur('curseurTemperature', minTemperature, maxTemperature, temperature, 51, true, modifTemperature);
        
        var checkbox = document.getElementById('normaliser');
        normaliser = checkbox.checked;
        checkbox = document.getElementById('log');
        yLog = checkbox.checked;
        var choix = document.getElementById('max');
        maxLambda = choix.value * 1E-6;
        
        context = dessin.getContext('2d');
        context.translate(0.5, 0.5); // moche avec flashcanvas et fillText...
        context.font = 'bold 12px sans-serif';
        initGradient();
        majCroquis();
    }
}
function initGradient() {
    var npix = imgdimx - 2 * espx;
    degrade = context.createLinearGradient(espx, 0, espx+npix, 0);
    // couleurs issues de
    // http://www.efg2.com/Lab/ScienceAndEngineering/Spectra.htm
    // et http://www.physics.sfasu.edu/astro/color.html
    // 360 et -: (0, 0, 0)
    // 380: (1/3, 0, 1/3)
    // 420: (1/3, 0, 1)
    // 440: (0, 0, 1)
    // 490: (0, 1, 1)
    // 510: (0, 1, 0)
    // 580: (1, 1, 0)
    // 645: (1, 0, 0)
    // 700: (1, 0, 0)
    // 820 et +: (0, 0, 0)
    var x;
    degrade.addColorStop(0, '#000000');
    x = lambdaVersP(360E-9);
    degrade.addColorStop(x, '#000000');
    x = lambdaVersP(380E-9);
    degrade.addColorStop(x, '#550055');
    x = lambdaVersP(420E-9);
    degrade.addColorStop(x, '#5500FF');
    x = lambdaVersP(440E-9);
    degrade.addColorStop(x, '#0000FF');
    x = lambdaVersP(490E-9);
    degrade.addColorStop(x, '#00FFFF');
    x = lambdaVersP(510E-9);
    degrade.addColorStop(x, '#00FF00');
    x = lambdaVersP(580E-9);
    degrade.addColorStop(x, '#FFFF00');
    x = lambdaVersP(645E-9);
    degrade.addColorStop(x, '#FF0000');
    x = lambdaVersP(700E-9);
    degrade.addColorStop(x, '#FF0000');
    x = lambdaVersP(820E-9);
    degrade.addColorStop(x, '#000000');
    degrade.addColorStop(1, '#000000');
}

function lambdaVersP(lambda) {
    return((lambda - minLambda) / (maxLambda - minLambda));
}

function majCroquis() {
    context.beginPath();
    context.fillStyle = 'white';
    context.fillRect(0, 0, imgdimx, imgdimx);
    
    context.strokeStyle = 'black';
    context.beginPath();
    var npix = imgdimx - 2 * espx;
    var hsurk = h/k;
    var umax;
    
    afficherTemperature();
    
    // lambda max
    var lambdamax = wien/temperature;
    p_lambda_max.innerHTML = '\u03BB<sub>max</sub> = ' + (new Number(1E6*lambdamax)).toPrecision(5) + ' \u03BCm';
    if (normaliser) {
        umax = Math.pow(lambdamax, -5)/(Math.exp(hsurk*c/(lambdamax*temperature)) - 1);
    } else {
        lambdamax = wien/maxTemperature;
        umax = Math.pow(lambdamax, -5)/(Math.exp(hsurk*c/(lambdamax*maxTemperature)) - 1);
    }
    
    // courbe
    context.save();
    context.beginPath();
    if (yLog)
        umax = Math.log(umax);
    var premier = false;
    for (var i=0; i<npix; i++) {
        var lambda = minLambda + ((maxLambda - minLambda)*i)/npix;
        var nu = c / lambda;
        var u = Math.pow(lambda, -5)/(Math.exp(hsurk*c/(lambda*temperature)) - 1);
        if (yLog)
            u = Math.log(u);
        if (u/umax > 1E-3) {
            var y1 = imgdimx - espy - 1 - Math.round((imgdimx - espy - 5)*(u/umax));
            if (!premier) {
                context.moveTo(espx + i, imgdimx - espy - 1);
                premier = true;
            }
            context.lineTo(espx + i, y1);
        }
    }
    context.lineTo(espx + npix - 1, imgdimx - espy - 1);
    context.closePath();
    context.clip();
    context.fillStyle = degrade;
    context.fillRect(espx, 5, npix, imgdimx - espy - 6);
    context.restore();
    
    // axes
    context.beginPath();
    context.strokeStyle = '#505050';
    
    context.moveTo(espx, 5);
    context.lineTo(espx, imgdimx - espy);
    
    context.moveTo(espx, 5);
    context.lineTo(espx - 2, 10);
    
    context.moveTo(espx, 5);
    context.lineTo(espx + 2, 10);
    
    context.moveTo(espx, imgdimx - espy);
    context.lineTo(imgdimx - espx, imgdimx - espy);
    
    context.moveTo(imgdimx - espx, imgdimx - espy);
    context.lineTo(imgdimx - espx - 5, imgdimx - espy - 2);
    
    context.moveTo(imgdimx - espx, imgdimx - espy);
    context.lineTo(imgdimx - espx - 5, imgdimx - espy + 2);
    
    context.stroke();
    
    context.fillStyle = 'black';
    context.textAlign = 'left';
    context.textBaseline = 'alphabetic';
    context.fillText('1', 2, 15);
    context.fillText('0', 2, imgdimx - espy);
    context.fillText((minLambda*1E6) + '\u03BCm', espx, imgdimx - 4);
    context.fillText((maxLambda*1E6) + '\u03BCm', imgdimx - 30, imgdimx - 4);
    context.fillText('\u03BB', imgdimx - espx + 2, imgdimx - espy);
    
    idTimeout = 0;
}

function afficherTemperature() {
    var texte = 'Température : ' + (new Number(temperature)).toFixed(0) + ' K';
    var element = document.getElementById('temperature');
    var first = element.firstChild;
    if (first == null) {
        element.appendChild(document.createTextNode(texte));
        return;
    } else
        first.nodeValue = texte;
}

function modifTemperature(e) {
    var id = e.target.id;
    if (id == 'curseurTemperature') {
        temperature = curseur_temperature.getValeur();
        afficherTemperature();
        if (idTimeout != 0)
            clearTimeout(idTimeout);
        idTimeout = setTimeout(majCroquis, 0);
    }
}

function modifNormaliser() {
    var checkbox = document.getElementById('normaliser');
    normaliser = checkbox.checked;
    majCroquis();
}

function modifLog() {
    var checkbox = document.getElementById('log');
    yLog = checkbox.checked;
    majCroquis();
}

function modifMax() {
    var choix = document.getElementById('max');
    maxLambda = choix.value * 1E-6;
    initGradient();
    majCroquis();
}
